@@ -1,11 +1,16 @@
 """Git merge implementation."""
 
-from typing import Optional
+from collections.abc import Sequence
+from typing import TYPE_CHECKING, Optional
 
-try:
+if TYPE_CHECKING:
     import merge3
-except ImportError:
-    merge3 = None  # type: ignore
+    from merge3 import SequenceMatcherProtocol
+else:
+    try:
+        import merge3
+    except ImportError:
+        merge3 = None  # type: ignore[assignment]
 
 from dulwich.attrs import GitAttributes
 from dulwich.config import Config
@@ -14,14 +19,20 @@ from dulwich.object_store import BaseObjectStore
 from dulwich.objects import S_ISGITLINK, Blob, Commit, Tree, is_blob, is_tree
 
 
-def make_merge3(*args, **kwargs) -> "merge3.Merge3":
+def make_merge3(
+    base: Sequence[bytes],
+    a: Sequence[bytes],
+    b: Sequence[bytes],
+    is_cherrypick: bool = False,
+    sequence_matcher: Optional[type["SequenceMatcherProtocol[bytes]"]] = None,
+) -> "merge3.Merge3[bytes]":
     """Return a Merge3 object, or raise ImportError if merge3 is not installed."""
     if merge3 is None:
         raise ImportError(
             "merge3 module is required for three-way merging. "
             "Install it with: pip install merge3"
         )
-    return merge3.Merge3(*args, **kwargs)
+    return merge3.Merge3(base, a, b, is_cherrypick, sequence_matcher)
 
 
 class MergeConflict(Exception):
@@ -39,7 +50,7 @@ class MergeConflict(Exception):
 
 
 def _can_merge_lines(
-    base_lines: list[bytes], a_lines: list[bytes], b_lines: list[bytes]
+    base_lines: Sequence[bytes], a_lines: Sequence[bytes], b_lines: Sequence[bytes]
 ) -> bool:
     """Check if lines can be merged without conflict."""
     # If one side is unchanged, we can take the other side
@@ -55,7 +66,7 @@ def _can_merge_lines(
 
 if merge3 is not None:
 
-    def _merge3_to_bytes(m: merge3.Merge3) -> bytes:
+    def _merge3_to_bytes(m: "merge3.Merge3[bytes]") -> bytes:
         """Convert merge3 result to bytes with conflict markers.
 
         Args:
@@ -64,8 +75,8 @@ if merge3 is not None:
         Returns:
             Merged content as bytes
         """
-        result = []
-        for group in m.merge_groups():
+        result: list[bytes] = []
+        for group in m.merge_groups():  # type: ignore[no-untyped-call,unused-ignore]
             if group[0] == "unchanged":
                 result.extend(group[1])
             elif group[0] == "a":
@@ -94,8 +105,8 @@ if merge3 is not None:
 
 
 def _merge_lines(
-    base_lines: list[bytes], a_lines: list[bytes], b_lines: list[bytes]
-) -> list[bytes]:
+    base_lines: Sequence[bytes], a_lines: Sequence[bytes], b_lines: Sequence[bytes]
+) -> Sequence[bytes]:
     """Merge lines when possible."""
     if base_lines == a_lines:
         return b_lines
@@ -291,20 +302,23 @@ class Merger:
         Returns:
             tuple of (merged_tree, list_of_conflicted_paths)
         """
-        conflicts = []
-        merged_entries = {}
+        conflicts: list[bytes] = []
+        merged_entries: dict[bytes, tuple[Optional[int], Optional[bytes]]] = {}
 
         # Get all paths from all trees
         all_paths = set()
 
         if base_tree:
             for entry in base_tree.items():
+                assert entry.path is not None
                 all_paths.add(entry.path)
 
         for entry in ours_tree.items():
+            assert entry.path is not None
             all_paths.add(entry.path)
 
         for entry in theirs_tree.items():
+            assert entry.path is not None
             all_paths.add(entry.path)
 
         # Process each path
